@@ -1,5 +1,6 @@
 #define QT_FEATURE_xcb 1
 #define QT_FEATURE_accessibility_atspi_bridge 1
+#define QT_FEATURE_angle_d3d11_qdtd -1
 #define QT_FEATURE_direct2d -1
 #define QT_FEATURE_directfb -1
 #define QT_FEATURE_egl 1
@@ -20,6 +21,7 @@
 #define QT_FEATURE_harfbuzz 1
 #define QT_FEATURE_ico 1
 #define QT_FEATURE_integrityfb -1
+#define QT_FEATURE_integrityhid -1
 #define QT_FEATURE_jpeg 1
 #define QT_FEATURE_libinput 1
 #define QT_FEATURE_libinput_axis_api 1
@@ -35,11 +37,13 @@
 #define QT_FEATURE_tslib 1
 #define QT_FEATURE_vnc 1
 #define QT_FEATURE_xcb_glx 1
-#define QT_FEATURE_xcb_render 1
-#define QT_FEATURE_xcb_sm 1
 #define QT_FEATURE_xlib 1
 #define QT_FEATURE_xcb_xlib 1
+#define QT_FEATURE_xcb_native_painting 1
+#define QT_FEATURE_xcb_render 1
+#define QT_FEATURE_xcb_sm 1
 #define QT_FEATURE_xinput2 1
 #define QT_FEATURE_xkb 1
 #define QT_FEATURE_xkbcommon_system 1
 #define QT_FEATURE_xkbcommon_evdev 1
+#define QT_FEATURE_xrender 1
