@@ -58,6 +58,10 @@
 #include <QtGui/qcolor.h>
 #include <QtGui/qpixmap.h>
 #include <QtGui/qimage.h>
+#include <QtGui/qregion.h>
+#include <QtGui/qvector2d.h>
+#include <QtGui/qvector3d.h>
+#include <QtGui/qvector4d.h>
 
 #ifdef QT_WIDGETS_LIB
 #include <QtGui/qicon.h>
@@ -79,9 +83,68 @@ namespace QTest
  */
 template<> inline char *toString(const QColor &color)
 {
-    return qstrdup(color.name().toLocal8Bit().constData());
+    return qstrdup(color.name(QColor::HexArgb).toLocal8Bit().constData());
 }
 
+template<> inline char *toString(const QRegion &region)
+{
+    QByteArray result = "QRegion(";
+    if (region.isNull()) {
+        result += "null";
+    } else if (region.isEmpty()) {
+        result += "empty";
+    } else {
+        const QVector<QRect> &rects = region.rects();
+        const int rectCount = rects.size();
+        if (rectCount > 1) {
+            result += QByteArray::number(rectCount);
+            result += " rectangles, ";
+        }
+        for (int i = 0; i < rectCount; ++i) {
+            if (i)
+                result += ", ";
+            const QRect &r = rects.at(i);
+            result += QByteArray::number(r.width());
+            result += 'x';
+            result += QByteArray::number(r.height());
+            if (r.x() >= 0)
+                result += '+';
+            result += QByteArray::number(r.x());
+            if (r.y() >= 0)
+                result += '+';
+            result += QByteArray::number(r.y());
+        }
+    }
+    result += ')';
+    return qstrdup(result.constData());
+}
+
+#ifndef QT_NO_VECTOR2D
+template<> inline char *toString(const QVector2D &v)
+{
+    QByteArray result = "QVector2D(" + QByteArray::number(double(v.x())) + ", "
+        + QByteArray::number(double(v.y())) + ')';
+    return qstrdup(result.constData());
+}
+#endif // !QT_NO_VECTOR2D
+#ifndef QT_NO_VECTOR3D
+template<> inline char *toString(const QVector3D &v)
+{
+    QByteArray result = "QVector3D(" + QByteArray::number(double(v.x())) + ", "
+        + QByteArray::number(double(v.y())) + ", " + QByteArray::number(double(v.z())) + ')';
+    return qstrdup(result.constData());
+}
+#endif // !QT_NO_VECTOR3D
+#ifndef QT_NO_VECTOR4D
+template<> inline char *toString(const QVector4D &v)
+{
+    QByteArray result = "QVector4D(" + QByteArray::number(double(v.x())) + ", "
+        + QByteArray::number(double(v.y())) + ", " + QByteArray::number(double(v.z()))
+        + ", " + QByteArray::number(double(v.w())) + ')';
+    return qstrdup(result.constData());
+}
+#endif // !QT_NO_VECTOR4D
+
 inline bool qCompare(QIcon const &t1, QIcon const &t2, const char *actual, const char *expected,
                     const char *file, int line)
 {
