@@ -43,6 +43,8 @@
 #include <QtWidgets/qtwidgetsglobal.h>
 #include <QtWidgets/qwidget.h>
 
+QT_REQUIRE_CONFIG(dialogbuttonbox);
+
 QT_BEGIN_NAMESPACE
 
 
@@ -53,7 +55,6 @@ class QDialogButtonBoxPrivate;
 class Q_WIDGETS_EXPORT QDialogButtonBox : public QWidget
 {
     Q_OBJECT
-    Q_FLAGS(StandardButtons)
     Q_PROPERTY(Qt::Orientation orientation READ orientation WRITE setOrientation)
     Q_PROPERTY(StandardButtons standardButtons READ standardButtons WRITE setStandardButtons)
     Q_PROPERTY(bool centerButtons READ centerButtons WRITE setCenterButtons)
@@ -104,13 +105,16 @@ public:
     };
 
     Q_DECLARE_FLAGS(StandardButtons, StandardButton)
+    Q_FLAG(StandardButtons)
 
     enum ButtonLayout {
-        // keep this in sync with QMessageBox::ButtonLayout and QPlatformDialogHelper::ButtonLayout
+        // keep this in sync with QPlatformDialogHelper::ButtonLayout
         WinLayout,
         MacLayout,
         KdeLayout,
-        GnomeLayout
+        GnomeLayout,
+        // MacModelessLayout,
+        AndroidLayout = GnomeLayout + 2 // ### Qt 6: reorder
     };
 
     QDialogButtonBox(QWidget *parent = Q_NULLPTR);
